@@ -320,24 +320,18 @@ class Cmd4Accessory
 
    setStoredValueForIndex( accTypeEnumIndex, value )
    {
-      if ( accTypeEnumIndex < 0 || accTypeEnumIndex > CMD4_ACC_TYPE_ENUM.EOL )
-      {
-         this.log.error ( chalk.red( `Error` ) + `: setStoredValue - Characteristic: ${ accTypeEnumIndex } for: ${ this.displayName } not known` );
-         this.log.error ( `Check your config.json file for this error` );
-         process.exit( 200 );
-      }
+      if ( accTypeEnumIndex < 0 || accTypeEnumIndex >= CMD4_ACC_TYPE_ENUM.EOL )
+         throw new Error( `setStoredValue - Characteristic index: ${ accTypeEnumIndex } not between 0 and ${ CMD4_ACC_TYPE_ENUM.EOL } for "${ this.displayName }"\nCheck your config.json file for unknown characteristic.` );
+
 
       this.storedValuesPerCharacteristic[ accTypeEnumIndex ] = value;
    }
 
    getStoredValueForIndex( accTypeEnumIndex )
    {
-      if ( accTypeEnumIndex < 0 || accTypeEnumIndex > CMD4_ACC_TYPE_ENUM.EOL )
-      {
-         this.log.error( `CMD4 Warning: getStoredValue - Characteristic: ${ accTypeEnumIndex } for: ${ this.displayName }` );
-         this.log.error( `Check your config.json file for this error` );
-         process.exit( 205 );
-      }
+      if ( accTypeEnumIndex < 0 || accTypeEnumIndex >= CMD4_ACC_TYPE_ENUM.EOL )
+         throw new Error( `getStoredValue - Characteristic index: ${ accTypeEnumIndex } not between 0 and ${ CMD4_ACC_TYPE_ENUM.EOL } for "${ this.displayName }"\nCheck your config.json file for unknown characteristic.` );
+
       return this.storedValuesPerCharacteristic[ accTypeEnumIndex ];
    }
 
@@ -444,40 +438,33 @@ class Cmd4Accessory
                   let ucValue = ucFirst( value );
                   accTypeEnumIndex = CMD4_ACC_TYPE_ENUM.properties.indexOfEnum( i => i.type === ucValue );
                   if ( accTypeEnumIndex < 0 )
-                  {
-                     this.log.error( chalk.red( `No such polling characteristic: ${ value } for: ${ this.displayName }` ) );
-                     process.exit( 210 );
-                  }
+                     throw new Error( `No such polling characteristic: "${ value }" for: "${ this.displayName }".` );
+
                   // We can do this as this is a new way to do things.
                   let storedValue = this.getStoredValueForIndex( accTypeEnumIndex );
                   if ( storedValue == undefined )
-                  {
-                     this.log.error( `Polling for: "${ value }" requested, but characteristic` +
-                                     `is not in your config.json file for: ${ this.displayName }` );
-                     process.exit( 211 );
-                  }
+                     throw new Error( `Polling for: "${ value }" requested, but characteristic is not in your config.json file for: "${ this.displayName }".` );
+
                   // This makes thinks nice down below.
                   valueToStore = storedValue;
                   break;
                }
                default:
                {
+                  // Is this still useful?
                   accTypeEnumIndex = CMD4_ACC_TYPE_ENUM.properties.indexOfEnum( i => i.type === ucKey );
 
                   if ( accTypeEnumIndex < 0  )
-                  {
-                    this.log.error( `OOPS: "${ key }" not found while parsing for characteristic polling. There something wrong with your config.json file?` );
-                    process.exit( 212 );
-                  }
+                     throw new Error( `OOPS: "${ key }" not found while parsing for characteristic polling. There something wrong with your config.json file?` );
+
                   valueToStore = value;
                }
             }
          }
+
          if ( accTypeEnumIndex == -1 )
-         {
-            this.log.error( `No characteristic found while parsing for characteristic polling. There something wrong with your config.json file?` );
-            process.exit( 213 );
-         }
+            throw new Error( `No characteristic found while parsing for characteristic polling of: "${ this.displayName }". There something wrong with your config.json file?` );
+
          if ( this.getStoredValueForIndex( accTypeEnumIndex ) == undefined )
          {
             this.log.warn( `Polling for: "${ key }" requested, but characteristic` );
@@ -662,16 +649,12 @@ class Cmd4Accessory
       for ( let key in definitions )
       {
          if ( characteristicProps[ key ] == undefined )
-         {
-            this.log.error( chalk.red( `Error` ) + `: props for key: ${ key } not in definition of: ${ type }` );
-            process.exit( 220 );
-         }
+            throw new Error( `props for key "${ key }" not in definition of "${ type }"` );
+
 
          if ( typeof characteristicProps[ key ] !=  typeof definitions[ key ] )
-         {
-            this.log.error( chalk.red( `Error` ) + `: props for key: ${ key } type: ${ typeof definitions[ key ] }  not equal to definition of: ${ typeof characteristicProps[ key ] }` );
-            process.exit( 221 );
-         }
+            throw new Error( `props for key "${ key }" type "${ typeof definitions[ key ] }" Not equal to definition of "${ typeof characteristicProps[ key ] }"` );
+
       }
 
       return rc;
@@ -1110,18 +1093,7 @@ class Cmd4Accessory
                     case constants.FAKEGATO_TYPE_AQUA_l:
                        break;
                     default:
-                       this.log.error( `Invalid fakegato eve type: ${ value }` );
-                       this.log.error( "It must be one of ( %s, %s, %s, %s, %s, %s, %s )",
-                          constants.FAKEGATO_TYPE_ENERGY_l,
-                          constants.FAKEGATO_TYPE_ROOM_l,
-                          constants.FAKEGATO_TYPE_WEATHER_l,
-                          constants.FAKEGATO_TYPE_DOOR_l,
-                          constants.FAKEGATO_TYPE_MOTION_l,
-                          constants.FAKEGATO_TYPE_THERMO_l,
-                          constants.FAKEGATO_TYPE_AQUA_l );
-                        this.log.error( `Check the Cmd4 README at ` );
-                        this.log.error( `https://github.com/simont77/fakegato-history` );
-                        process.exit( 225 );
+                       throw new Error( `Invalid fakegato eve type: "${ value }". It must be one of ( ${ constants.FAKEGATO_TYPE_ENERGY_l }, ${ constants.FAKEGATO_TYPE_ROOM_l }, ${ constants.FAKEGATO_TYPE_WEATHER_l }, ${ constants.FAKEGATO_TYPE_DOOR_l }, ${ constants.FAKEGATO_TYPE_MOTION_l }, ${ constants.FAKEGATO_TYPE_THERMO_l }, ${ constants.FAKEGATO_TYPE_AQUA_l } ). Check the Cmd4 README at: "https://github.com/simont77/fakegato-history".` );
                 }
                 break;
              case constants.STORAGE:
@@ -1153,31 +1125,20 @@ class Cmd4Accessory
                 // and if it is not 0 for not used.
                 if ( ucValue != "0" )
                 {
-                   if ( accTypeEnumIndex <0 )
-                   {
-                      this.log.error( chalk.red( `Error` ) + `Invalid characteristic: ${ value } for` +
-                                      ` fakegato to log of: ${ key }` );
-                      process.exit( 216 );
-                   }
+                   if ( accTypeEnumIndex < 0 )
+                      throw new Error( `Invalid characteristic "${ value }" for fakegato to log of "${ key }".` );
+
 
                    // Make sure the characteristic is being polled so I do not get any more tickets
                    // as to why the value is not changing.
-                   if ( settings.arrayOfPollingCharacteristics.filter( entry => entry.accessory.UUID == this.UUID &&
-                                                                       entry.accTypeEnumIndex == accTypeEnumIndex
-                                                                     ).length == 0
-                      )
-                   {
-                       this.log.error( chalk.red( 'Error' ) + `: Characteristic: ${ value } for fakegato` +
-                                       ` to log of: ${ key } is not being polled.\n` );
-                       this.log.error( `History can not be updated continiously.` );
-                       process.exit( 217 );
-                   }
+                   if ( this.queue.isCharacteristicPolled( accTypeEnumIndex, this.queue, this ) == false )
+                       throw new Error(`Characteristic: "${ value }" for fakegato to log of "${ key }" is not being polled.\nHistory can not be updated continiously.` );
+
                 }
                 break;
              }
              default:
-                this.log.error( `Invalid fakegato key: ${ key } in json.config for: ${ this.displayName }` );
-                process.exit( 218 );
+                throw new Error( `Invalid fakegato key: "${ key }" in json.config for: "${ this.displayName }".` );
           }
       }
 
@@ -1225,11 +1186,9 @@ class Cmd4Accessory
 
    validateStateCmd( state_cmd )
    {
-      if ( ! state_cmd )
-      {
-         this.log.error( chalk.red( `No state_cmd for: ${ this.displayName }` ) );
-         process.exit( 230 );
-      }
+      if ( typeof state_cmd != "string" )
+         throw new Error( `No state_cmd for: "${ this.displayName }".` );
+
 
       // Split the state_cmd into words.
       let cmdArray = state_cmd.match(/\S+/gi);
@@ -1260,7 +1219,7 @@ class Cmd4Accessory
 
             } catch ( e ) {
                // It isn't accessible
-               this.log.warn( `The script ${ checkFile } does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
+               throw new Error( `The file: "${ checkFile }" does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
             }
          }
          // Purposely fall through to check the command as well
@@ -1282,12 +1241,12 @@ class Cmd4Accessory
                   fs.accessSync( checkCmd, fs.F_OK );
                   // File exists - OK
                } catch ( e ) {
-                  this.log.warn( `The file ${ checkCmd } does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
+                  throw new Error( `The file: "${ checkCmd }" does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
                }
             } else
             {
                if ( ! commandExistsSync( checkCmd ) )
-                  this.log.warn( `The command ${ checkCmd } does not exist or is not in your path, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
+                  throw new Error( `The file: "${ checkCmd }" does not exist, It is highly likely the state_cmd will fail. Hint: Do not use wildcards that would normally be expanded by a shell.` );
             }
             break;
          }
@@ -1303,10 +1262,8 @@ class Cmd4Accessory
       let accTypeEnumIndex = CMD4_ACC_TYPE_ENUM.properties.indexOfEnum( i => i.type === ucKey );
 
       if ( accTypeEnumIndex < 0 )
-      {
-         this.log.error( `OOPS: "${ key }" not found for parsing key for Characteristics. There something wrong with your config.json file?` );
-         process.exit( 235 );
-      }
+         throw new Error( `OOPS: "${ key }" not found for parsing characteristics in: "${ this.displayName }".` );
+
 
       // Do not update the stored values as it is being restored from cache
       if ( parseConfigShouldUseCharacteristicValues == false )
@@ -1315,17 +1272,10 @@ class Cmd4Accessory
       if ( Object.keys( CMD4_ACC_TYPE_ENUM.properties[ accTypeEnumIndex ].validValues ).length > 0 )
       {
          // Even if outputConsts is not set, just in case, transpose it anyway.
-         //var transposed = { [ constants.VALUE_lv ]: value, [ constants.RC_lv ]: true, [ constants.MSG_lv ]: "" };
          value = transposeConstantToValidValue( CMD4_ACC_TYPE_ENUM.properties, accTypeEnumIndex, value ) ;
-
-         //if ( transposed.rc == false )
-         //   this.log.warn( `${ this.displayName }: ${ transposed.msg }` );
-
-         //value = transposed.value;
       }
 
       // Return the appropriate type, by seeing what it is defined as in Homebridge,
-      //value = this.characteristicValueToItsProperType( this, accTypeEnumIndex, value );
       let properValue = CMD4_ACC_TYPE_ENUM.properties[ accTypeEnumIndex ].stringConversionFunction( value );
       if ( properValue == undefined )
       {
@@ -1357,10 +1307,8 @@ class Cmd4Accessory
             let required = requires[ key ] ;
 
             if ( typeof required != "string" )
-            {
-               this.log.error( `Requires definition: ${ required }  must be a string` );
-               process.exit( 240 );
-            }
+               throw new Error( `Requires definition: "${ required }"  must be a string.` );
+
 
             if ( cmd4Dbg ) this.log.debug( `Requiring ${ required }` );
 
@@ -1368,8 +1316,7 @@ class Cmd4Accessory
          }
          return;
       }
-      this.log.error( `Requires must be an array of/or list of key/value pairs: ${ requires }` );
-      process.exit( 241 );
+      throw new Error( `Requires must be an array of/or list of key/value pairs: "${ requires }".` );
    }
 
    processConstants( constantArg )
@@ -1378,7 +1325,7 @@ class Cmd4Accessory
       {
          for ( let constant in constantArg )
             this.processConstants( constantArg[ constant ] );
- 
+
          return;
       }
       if ( isJSON( constantArg ) )
@@ -1389,16 +1336,12 @@ class Cmd4Accessory
             let keyToAdd = key ;
             let valueToAdd = constantArg[ key ] ;
             if ( ! keyToAdd.startsWith( "${" ) )
-            {
-               this.log.error( `Constant definition for: ${ keyToAdd } must start with "\${" for clarity.` );
-               process.exit( 245 );
-            }
+               throw new Error( `Constant definition for: "${ keyToAdd }" must start with "\${" for clarity.` );
+
 
             if ( ! keyToAdd.endsWith( "}" ) )
-            {
-               this.log.error( `Constant definition for: ${ keyToAdd } must end with "}" for clarity.` );
-               process.exit( 246 );
-            }
+               throw new Error( `Constant definition for: "${ keyToAdd }" must end with "}" for clarity.` );
+
 
             // remove any leading and trailing single quotes
             // so that using it for replacement will be easier.
@@ -1410,8 +1353,7 @@ class Cmd4Accessory
         return;
      }
 
-     this.log.error( `Constants must be an array of/or list of key/value pairs: ${ constantArg }` );
-     process.exit( 247 );
+     throw new Error( `Constants must be an array of/or list of key/value pairs: "${ constantArg }".` );
    }
 
    processVariables( variables )
@@ -1431,16 +1373,10 @@ class Cmd4Accessory
             let keyToAdd = key ;
             let valueToAdd = variables[ key ] ;
             if ( ! keyToAdd.startsWith( "${" ) )
-            {
-               this.log.error( `Variable definition for: ${ keyToAdd } must start with "\${" for clarity.` );
-               process.exit( 250 );
-            }
+               throw new Error( `Variable definition for: "${ keyToAdd }" must start with "\${" for clarity.` );
 
             if ( ! keyToAdd.endsWith( "}" ) )
-            {
-               this.log.error( `Variable definition for: ${ keyToAdd } must end with "}" for clarity.` );
-               process.exit( 251 );
-            }
+               throw new Error( `Variable definition for: "${ keyToAdd }" must end with "}" for clarity.` );
 
             // remove any leading and trailing single quotes
             // so that using it for replacement will be easier.
@@ -1455,8 +1391,7 @@ class Cmd4Accessory
          return;
       }
 
-      this.log.error( `Variables must be an array of/or list of key/value pairs: ${ variables }` );
-      process.exit( 252 );
+      throw new Error( `Variables must be an array of/or list of key/value pairs: "${ variables }".` );
    }
 
    accessoryTypeConfigToCmd4Accessories( config, parentInfo )
@@ -1487,16 +1422,7 @@ class Cmd4Accessory
    processURL( url )
    {
       if ( typeof url != "string" )
-      {
-         this.log.error( `url must be a string: ${ url }` );
-         process.exit( 255 );
-      }
-
-      if ( this.url !== undefined )
-      {
-         this.log.error( `url is already defined as: ${ this.url } for: ${ url }` );
-         process.exit( 256 );
-      }
+         throw new Error( `url must be a string: "${ url }".` );
 
       this.url = this.replaceConstantsInString( url );
    }
@@ -1557,10 +1483,7 @@ class Cmd4Accessory
                this.category = this.api.hap.Categories[ String( value ).toUpperCase( ) ];
 
                if ( ! this.category )
-               {
-                  this.log.error( `Category specified: ${ value } is not a valid homebridge category.` );
-                  process.exit( 260 );
-               }
+                  throw new Error( `Category specified: "${ value }" is not a valid homebridge category for: "${ this.displayName }".` );
 
                break;
             case constants.PUBLISHEXTERNALLY:
@@ -1636,11 +1559,8 @@ class Cmd4Accessory
             {
                let queue = queueExists( value );
                if ( queue == undefined )
-               {
-                  this.log.error( chalk.red( `"QueueType" must be defined first for: "queue"` ) );
+                  throw new Error( `"QueueType" must be defined first for queue "${ value }" in: "${ this.displayName }"` );
 
-                  process.exit( 261 ) ;
-               }
                this.queue = queue;
 
                break;
@@ -1749,27 +1669,19 @@ class Cmd4Accessory
       if ( cmd4Mode != null && this.polling )
       {
          if ( cmd4Mode == "Demo" )
-         {
-            //process.exit(123);
             throw new Error("Demo mode is achieved when there are no polling entries in your config.json");
-         }
+
          this.log.warn( `Cmd4 has been simplified and optimized as per: https://git.io/JtMGR.` );
          this.log.warn( `To remove this message, just remove "Cmd4_Mode" from your config.json` );
       }
 
       if ( trueTypeOf( this.type ) != String )
-      {
-          this.log.error( chalk.red( `Error` ) + `: No device type given for: ${ this.displayName }` );
-         process.exit( 262 );
-      }
+          throw new Error( `No device type given in: "${ this.displayName }"` );
 
       this.ucType = ucFirst( this.type );
       this.typeIndex = CMD4_DEVICE_TYPE_ENUM.properties.indexOfEnum( i => i.deviceName === this.ucType );
       if ( this.typeIndex < 0 )
-      {
-          this.log.error( chalk.red( `Error` ) + `: Unknown device type: ${ this.type } for: ${ this.displayName }` );
-         process.exit( 263 );
-      }
+          throw new Error( `Unknown device type: "${ this.type }" given in: "${ this.displayName }".` );
 
       // Create a subType to delimit services with multiple accessories of
       // the same type and possibly the same accessory.name.
@@ -1785,11 +1697,8 @@ class Cmd4Accessory
       // State_cmd is only required when polling is enabled.
       if ( this.polling )
       {
-         if ( ! this.validateStateCmd( this.state_cmd ) )
-         {
-            this.log.error( chalk.red( `Error` ) + `: state_cmd: "${ this.state_cmd }" is invalid for: ${ this.displayName }` );
-            process.exit( 264 );
-         }
+         // throws its own exceptio if it fails
+         this.validateStateCmd( this.state_cmd );
       } else
       {
          // Added accessories like input have no polled configuristics and therefore don't
@@ -1836,9 +1745,7 @@ class Cmd4Accessory
             if ( cmd4Dbg ) this.log.debug( `Polling config is Default Polling. Nothing to check for unset polling characteristics` );
             return;
          case String:
-            this.log.error( `Unknown type for Polling ${ pollingType }` );
-            process.exit( 266 );
-            return;
+            throw new Error( `Unknown type for Polling "${ this.polling }" given in: "${ this.displayName }".` );
          case Array:
             break;
          case Object:
@@ -1848,9 +1755,7 @@ class Cmd4Accessory
             this.polling = [ this.polling ];
             return;
          default:
-            this.log.error( `Do not know how to handle polling type of: ${ pollingType }` );
-            process.exit( 267 );
-            return;
+            throw new Error( `Do not know how to handle polling type of: "${ pollingType }" for: "${ this.displayName }".` );
       }
 
       // We need to check for removed characteristics in the config
@@ -2010,11 +1915,7 @@ class Cmd4Accessory
                   case constants.QUEUE:
                   {
                      if ( this.queue && this.queue.queueName != value )
-                     {
-                         this.log.error( chalk.red( `Already defined Priority Polling Queue "${ this.queue.queueName }" for ${ this.displayName } cannot be redefined.` ) );
-                         throw new Error('Already defined Priority Polling Queue' );
-                         // process.exit( 208 );
-                     }
+                        throw new Error( chalk.red( `Already defined Priority Polling Queue "${ this.queue.queueName }" for ${ this.displayName } cannot be redefined.` ) );
 
                      this.queue = addQueue( this.log, value, constants.QUEUETYPE_WORM );
 
@@ -2026,18 +1927,13 @@ class Cmd4Accessory
                      let ucValue = ucFirst( value );
                      accTypeEnumIndex = CMD4_ACC_TYPE_ENUM.properties.indexOfEnum( i => i.type === ucValue );
                      if ( accTypeEnumIndex < 0 )
-                     {
-                        this.log.error( chalk.red( `No such polling characteristic: ${ value } for: ${ accessory.displayName }` ) );
-                        process.exit( 261 );
-                     }
+                        throw new Error( `No such polling characteristic: "${ value }" for: "${ this.displayName }".` );
+
                      characteristicString = CMD4_ACC_TYPE_ENUM.properties[ accTypeEnumIndex ].type;
                      // We can do this as this is a new way to do things.
                      if ( this.getStoredValueForIndex( accTypeEnumIndex ) == undefined )
-                     {
-                        this.log.error( `Polling for: "${ value }" requested, but characteristic` +
-                                        ` is not in your config.json file for: ${ this.displayName }` );
-                        process.exit( 262 );
-                     }
+                        throw new Error( `Polling for: "${ value }" requested, but characteristic is not in your config.json file for: "${ this.displayName }".` );
+
                      break;
                   }
                   case constants.QUEUETYPES:
@@ -2052,16 +1948,12 @@ class Cmd4Accessory
                      // The key must be a characteristic property
                      // but first check if one has already been defined as we can only handle one at a time.
                      if ( accTypeEnumIndex != -1 )
-                     {
-                        this.log.error( chalk.red( `Error` ) + `: For charateristic polling, you can only define one characteristic per array item.\nCannot add "${ ucKey }" as "${ characteristicString }" is already defined for: ${ accessory.displayName }` );
-                        process.exit( 263 );
-                     }
+                        throw new Error( `For charateristic polling, you can only define one characteristic per array item.\nCannot add "${ ucKey }" as "${ characteristicString }" is already defined for: ${ accessory.displayName }` );
+
                      accTypeEnumIndex = CMD4_ACC_TYPE_ENUM.properties.indexOfEnum( i => i.type === ucKey );
                      if ( accTypeEnumIndex < 0 )
-                     {
-                        this.log.error( chalk.red( `Error` ) + ` No such polling characteristic: ${ key } for: ${ accessory.displayName }` );
-                        process.exit( 264 );
-                     }
+                        throw new Error( `No such polling characteristic: "${ key }" for: "${ accessory.displayName }".` );
+
                   }
                }
             }
@@ -2176,9 +2068,7 @@ function checkAccessoryForDuplicateUUID( accessory, UUID )
             if ( accessory.name == existingAccessory.name )
                accessory.log.error( chalk.red( `Duplicate accessory names can cause this issue.` ) );
 
-            accessory.log.error( chalk.red( `It is wiser to define the second accessory in a different bridge.` ) );
-
-            process.exit( 270 );
+            throw new Error( `It is wiser to define the second accessory in a different bridge.` );
          }
       }
       // Check for duplicates in Added accessories.
